import logging

# Root logger
logger = logging.getLogger("Artec")
logger.setLevel(logging.INFO)

# Handlers (sub-loggers)
file_handler = logging.FileHandler("log.txt", delay=True, mode="w")
console_handler = logging.StreamHandler()

# Formatters
file_handler.setFormatter(
    logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
)
console_handler.setFormatter(logging.Formatter("> %(message)s."))

# Levels
file_handler.setLevel(logging.INFO)

# Add the Handlers
logger.addHandler(file_handler)
logger.addHandler(console_handler)



from . import argparser as parser
from . import boiler
from . import logging, console_handler
from .temp import * 

__app_name__ = "Artec"
__version__ = "0.3.1"
__desc__ = "Creates a configurable python project \
    template in a given directory."

ERROR_LOGING = {
    0: logging.FATAL,
    1: logging.ERROR,
    2: logging.WARN,
    3: logging.INFO,
}


def main():
    args = parser.main_args(__version__)

    console_handler.setLevel(ERROR_LOGING[args.verbose])
    builder = boiler.boiler_builder(
        args.source, args.target, args.template, args.git
    )

    builder.build()


if __name__ == "__main__":
    main()
    



from . import logger


class NotJsonFile(FileNotFoundError):
    def __init__(self) -> None:
        super().__init__("> Provided Source isn't valid JSON file")
        self.errno = 101
        logger.warning("Provided Source isn't valid JSON file")
        logger.info("Reverting to default structure")

    def __str__(self):
        return "> Provided Source isn't valid JSON file"


class NoSource(ValueError):
    def __init__(self) -> None:
        super().__init__("> No Source Provided")
        self.errno = 102
        logger.warning("No Source Provided")
        logger.info("Using default structure")

    def __str__(self):
        return "> No Source Provided"


class NotValidJson(KeyError):
    def __init__(self) -> None:
        super().__init__("> Provided Json file format is incorrect")
        self.errno = 103
        print(self)
        logger.warning("Provided Json file format is incorrect")
        logger.error("Artec shut down unexpectedly.")

    def __str__(self):
        return "> Provided Json file format is incorrect"


class InValidTemplate(KeyError):
    def __init__(self) -> None:
        super().__init__("> Provided Template argument is invalid")
        self.errno = 104
        logger.warning("Provided Template argument is invalid")
        logger.error("Artec shut down unexpectedly.")

    def __str__(self):
        return "> Provided Template argument is invalid"


class GitError(OSError):
    def __init__(self , e = None) -> None:
        super().__init__("> Git has encountered a problem")
        self.errno = 105
        logger.warning("Git has encountered a problem")
        logger.error(e)
        logger.error("Artec shut down unexpectedly.")

    def __str__(self):
        return "> Git has encountered a problem"



from git import Repo
from . import logger
from os import listdir



class repository(Repo):

    def __init__(self, path, name) -> Repo:
        self.name = name
        self.repo_path = path
        self.repo = Repo.init(path)
        logger.info("Creating Git Repository")
    
    def add(self):
        files = listdir(self.repo_path)
        files.remove('.git')
        self.repo.index.add(files)
        logger.info("Adding the skeleton structure")
        self.repo.index.commit(f"{self.name} : initial commit.")
        logger.info("Committing the files")
            



import unittest
from artec.argparser import Parser, Namespace

VAILD_JSON = "test\\resources\\structure.json"
INVAILD_JSON = "test\\resources\\structure.txt"


class TestParser(unittest.TestCase):
    def test_init(self):
        parser = Parser("1.0.0")
        self.assertEqual(parser.prog, "Artec")
        self.assertEqual(parser.appVersion, "1.0.0")

    def test_setup(self):
        parser = Parser("1.0.0")
        parser.setup()
        self.assertIn("-h", parser.__dict__["_option_string_actions"])
        self.assertIn("-s", parser.__dict__["_option_string_actions"])
        self.assertIn("-t", parser.__dict__["_option_string_actions"])
        self.assertIn("-o", parser.__dict__["_option_string_actions"])
        self.assertIn("-ls", parser.__dict__["_option_string_actions"])
        self.assertIn("-v", parser.__dict__["_option_string_actions"])
        self.assertIn("-g", parser.__dict__["_option_string_actions"])
        self.assertIn("-V", parser.__dict__["_option_string_actions"])

    def test_main_args(self):
        parser = Parser("1.0.0")
        parser.setup()
        args = parser.parse_args(["-t", "target", "-o", "fake", "-v"])
        self.assertIsInstance(args, Namespace)
        self.assertIn("target", args.__dict__)
        self.assertIn("verbose", args.__dict__)

    def test_parse_without_args(self):
        parser = Parser("1.0.0")
        with self.assertRaises(SystemExit):
            parser.parse_args()

    def test_parse_with_source(self):
        parser = Parser("1.0.0")
        parser.setup()
        args = parser.parse_args(["-s", VAILD_JSON, "-o", "target"])
        self.assertIn("source", args.__dict__)
        self.assertEqual(args.source, VAILD_JSON)
        self.assertIn("target", args.__dict__)
        self.assertEqual(args.target, "target")

    def test_parse_with_template(self):
        parser = Parser("1.0.0")
        parser.setup()
        args = parser.parse_args(["-t", "python", "-o", "target"])
        self.assertIn("template", args.__dict__)
        self.assertEqual(args.template, "python")
        self.assertIn("target", args.__dict__)
        self.assertEqual(args.target, "target")


if __name__ == "__main__":
    pass



"""
	Main module core.
	#3 Step
"""
import json
import os
from pathlib import Path
from . import exceptions as ex
from .temp import *
from . import repo 
from . import logger 


class boiler_builder:
    def __init__(
        self,
        source: str = None,
        target: str = None,
        template: str = None,
        git=False,
    ) -> None:
        self.target = target
        self.git = git
        self.home_dir = os.path.abspath(os.path.curdir)

        if template is None:
            self.structure = self._source(source)
        else:
            self.structure = self._source_temp(template.lower())

    def _source_temp(self, template) -> list[dict[str, str]]:
        try:
            if template in templates:
                structure = format_project_structure(templates[template], self.target)

            else:
                raise ex.InValidTemplate()

        except ex.InValidTemplate:
            structure = format_project_structure(templates["python"], self.target)
        return structure

    def _source(self, source) -> list[dict[str, str]]:
        try:
            if source is None:
                raise ex.NoSource()
            if os.path.isfile(source) and source.endswith(".json"):
                with open(source, "rt", encoding="utf-8") as file_data:
                    structure = format_project_structure(json.load(file_data),
                        self.target
                    )
            else:
                raise ex.NotJsonFile()

        except Exception:
            structure = format_project_structure(templates["python"], self.target)

        return structure

    def build(self):
        print("> Creating folder structure: {}\n".format(self.target))

        for folder in self.structure['folders']:
            folder_path = os.path.join(self.target, folder)
            self._make_folder(folder_path)
            print("Created: {}".format(folder_path))

        for file_path in self.structure['files']:
            file_path = os.path.join(self.target, file_path)
            self._make_file(file_path)
            print("Created: {}".format(file_path))

        print()

        if self.git:
            try:
                Repo = repo.repository(os.path.join(self.home_dir, self.target), self.target)
                Repo.add()
                    
            except Exception as e :
                ex.GitError(e)

    def _make_file(self, path):
        """Create an empty file in a given directory"""
        open(path, "a").close()

    def _make_folder(self, path):
        """Create an empty directory"""
        os.makedirs(path, exist_ok=True)


if __name__ == "__main__":
    pass



from . import exceptions as ex


class static_list(list):
    def format(self, name):
        try:
            for _ in self:
                for i, j in _.items():
                    _[i] = j.format(name)
            return self
        except AttributeError:
            raise ex.NotValidJson()


PYTHON = static_list(
    [
        {"folder": "{}"},
        {"file": "{}/__init__.py"},
        {"folder": "test"},
        {"file": "test/__init__.py"},
        {"file": "README.md"},
        {"file": "LICENSE"},
        {"file": "setup.py"},
        {"file": "setup.cfg"},
        {"file": "pyproject.toml"},
    ]
)

FLASK = static_list(
    [
        {"folder": "{}"},
        {"file": "{}/__init__.py"},
        {"file": "{}/db.py"},
        {"file": "{}/schema.py"},
        {"file": "{}/auth.py"},
        {"file": "{}/blog.py"},
        {"folder": "{}/templates"},
        {"file": "{}/templates/base.html"},
        {"folder": "{}/templates/auth/"},
        {"file": "{}/templates/auth/login.html"},
        {"file": "{}/templates/auth/register.html"},
        {"folder": "{}/blog"},
        {"file": "{}/blog/create.html"},
        {"file": "{}/blog/index.html"},
        {"file": "{}/blog/update.html"},
        {"folder": "{}/static"},
        {"file": "{}/static/style.css"},
        {"folder": "test"},
        {"file": "test/__init__.py"},
        {"file": "test/conftest.py"},
        {"file": "test/data.sql"},
        {"file": "test/test_db.py"},
        {"file": "test/test_auth.py"},
        {"file": "test/test_blog.py"},
        {"file": "README.md"},
        {"file": "LICENSE"},
        {"file": "setup.py"},
        {"file": "pyproject.toml"},
    ]
)

NODE_JS = static_list(
    [
        {"folder": "src"},
        {"folder": "src/api"},
        {"folder": "src/api/controllers"},
        {"folder": "src/api/controllers/user"},
        {"folder": "src/api/controllers/user/auth"},
        {"file": "src/api/controllers/user/auth/forgot-password.js"},
        {"file": "src/api/controllers/user/auth/login.js"},
        {"file": "src/api/controllers/user/auth/logout.js"},
        {"file": "src/api/controllers/user/auth/refresh-token.js"},
        {"file": "src/api/controllers/user/auth/register.js"},
        {"file": "src/api/controllers/user/auth/send-verification-code.js"},
        {"file": "src/api/controllers/user/auth/verify-email.js"},
        {"folder": "src/api/controllers/user/edit"},
        {"file": "src/api/controllers/user/edit/change-password.js"},
        {"file": "src/api/controllers/user/edit/edit-user.js"},
        {"file": "src/api/controllers/user/delete-user.js"},
        {"file": "src/api/controllers/user/get-user.js"},
        {"file": "src/api/controllers/user/index.js"},
        {"folder": "src/api/middlewares"},
        {"folder": "src/api/middlewares/auth"},
        {"file": "src/api/middlewares/auth/check-auth.js"},
        {"file": "src/api/middlewares/auth/check-authority.js"},
        {"file": "src/api/middlewares/image-upload.js"},
        {"file": "src/api/middlewares/index.js"},
        {"file": "src/api/middlewares/object-id-control.js"},
        {"file": "src/api/middlewares/rate-limiter.js"},
        {"folder": "src/api/routes"},
        {"file": "src/api/routes/index.js"},
        {"file": "src/api/routes/user.js"},
        {"folder": "src/api/validators"},
        {"file": "src/api/validators/index.js"},
        {"file": "src/api/validators/user.validator.js"},
        {"folder": "src/config"},
        {"file": "src/config/index.js"},
        {"folder": "src/loaders"},
        {"file": "src/loaders/index.js"},
        {"file": "src/loaders/express.js"},
        {"file": "src/loaders/mongoose.js"},
        {"folder": "src/models"},
        {"file": "src/models/index.js"},
        {"file": "src/models/log.js"},
        {"file": "src/models/token.js"},
        {"file": "src/models/user.js"},
        {"folder": "src/utils"},
        {"folder": "src/utils/helpers"},
        {"file": "src/utils/helpers/error-helper.js"},
        {"file": "src/utils/helpers/generate-random-code.js"},
        {"file": "src/utils/helpers/ip-helper.js"},
        {"file": "src/utils/helpers/jwt-token-helper.js"},
        {"file": "src/utils/helpers/local-text-helper.js"},
        {"folder": "src/utils/lang"},
        {"file": "src/utils/lang/en.json"},
        {"file": "src/utils/lang/get-text.json"},
        {"file": "src/utils/lang/tr.json"},
        {"file": "src/utils/index.js"},
        {"file": "src/utils/logger.js"},
        {"file": "src/utils/send-code-to-email.js"},
        {"file": "src/app.js"},
        {"file": ".env.sample"},
        {"file": "README.md"},
        {"file": "LICENSE"},
        {"file": "package.json"},
        {"file": "package-lock.json"},
    ]
)

templates = {
    "python": PYTHON,
    "node.js": NODE_JS,
    "flask": FLASK,
}



from os import listdir, path
from .. import exceptions as ex
from json import load

DIR = path.dirname(path.abspath(__file__))


def list_available_templates() -> str :
    all_templates = "Available templates: \n\n"
    all_templates += "\n".join(
        [
            f"> {key.title()}\t" for key in templates.keys()
        ]
        )
    return all_templates

def format_project_structure(template, project_name):
    formatted_structure = {k: [] for k in template.keys()}  
    try:
        for folder in template['folders']:
            formatted_structure['folders'].append(folder.format(project_name))

        for file_path in template['files']:
            formatted_structure['files'].append(file_path.format(project_name))
    except Exception :
        raise ex.NotValidJson()
        
    return formatted_structure

def open_load(file_path):
    with open(path.join(DIR, file_path), "rt") as file:
        data = load(file)

    return data



templates = {
    filename.removesuffix(".json"): open_load(path.join(DIR, filename))  
    for filename in listdir(DIR) 
    if filename.endswith(".json")
}




"""
	Parser class is a wrapper for easy access of argparse module v2.1.0
"""
from argparse import (
    ArgumentParser,
    Namespace,
    RawTextHelpFormatter,
    _VersionAction,
)
from .temp import *
import sys


class list_templates(_VersionAction):
    def __call__(self, parser: ArgumentParser, *args, **kwargs) -> None:
        formatter = parser._get_formatter()
        formatter.add_text(
            list_available_templates()
        )
        parser._print_message(formatter.format_help(), sys.stdout)
        parser.exit()


class Parser(ArgumentParser):
    def __init__(self, appVersion):
        self.appVersion = appVersion
        prog = "Artec"
        usage = "artec [OPTIONS] -o [DEST] "
        description = "Artec is a python application that creates a configurable python project template in a given directory."
        epilog = "Examples:\n\tartec -h\n\tartec -o dest\
            \n\tartec -o dest -t python \n\tartec -o dest -s structure.json\
             \n\tartec -o dest -s structure.json -v"
        super().__init__(
            prog,
            usage,
            description,
            epilog,
            formatter_class=RawTextHelpFormatter,
        )

    def setup(self):
        group = self.add_mutually_exclusive_group()
        
        group.add_argument(
            "-s",
            "--source-file",
            dest="source",
            help="Source JSON file containing structure to be created",
            type=str,
            required=False,
        )

        group.add_argument(
            "-t",
            "--template",
            dest="template",
            help="Uses ready-made templates.",
            required=False,
        )


        self.add_argument(
            "-o",
            "--output-directory",
            dest="target",
            help="Target output path where the structure will be created",
            type=str,
        )

        self.add_argument(
            "-i",
            "--interactuve",
            dest="interactive",
            help="Runs artec in interactive mode.",
            action="store_true",
        )

        self.add_argument(
            "-ls",
            "--list-template",
            help="lists all ready-made templates.",
            action=list_templates,
        )

        self.add_argument(
            "-v",
            "--verbose",
            dest="verbose",
            help="Runs Artec in verbose mode.",
            action="count",
            default=0,
            required=False,
        )

        self.add_argument(
            "-g",
            "--git-init",
            dest="git",
            help="Creates a git Repo for the project.",
            action="store_true",
            required=False,
        )

        self.add_argument(
            "-V",
            "--version",
            help="Display current version of Artec",
            action="version",
            version=f"{self.prog} {self.appVersion}",
        )


def main_args(appVersion) -> Namespace:
    parser = Parser(appVersion)
    parser.setup()
    args = parser.parse_args()

    if args.interactive:
        print("Running Artec in interactive mode ....")  
    else:
        if not args.target:
            parser.error("The '-o' argument is required when not in interactive mode.")

    return args


if __name__ == "__main__":
    pass



import unittest
import json
import git
from pathlib import Path
from artec.boiler import boiler_builder

from artec.temp import *

VAILD_JSON = Path("./test/resources/structure.json")
INVAILD_JSON = Path("./test/resources/structure.txt")


class TestBoilerBuilder(unittest.TestCase):
    def test_init(self):
        builder = boiler_builder(
            source="structure.json", target="target"
        )
        self.assertEqual(builder.target, "target")
        self.assertIsInstance(builder.structure, dict)

    def test_source_temp(self):
        builder = boiler_builder(
            template="python", target="target"
        )
        self.assertEqual(
            builder.structure, format_project_structure(templates["python"], "target")
        )

    def test_source(self):
        with open(VAILD_JSON, "rt", encoding="utf-8") as file_data:
            structure = format_project_structure(json.load(file_data),"target")
        builder = boiler_builder(
            source=VAILD_JSON, target="target"
        )
        self.assertEqual(builder.structure, structure)

    def test_build(self):
        builder = boiler_builder(
            source=VAILD_JSON, target="target"
        )
        builder.build()
        self.assertTrue(Path("target/test").is_dir())
        self.assertTrue(Path("target/README.md").is_file())

    def test_git(self): 
        builder = boiler_builder(
            template="python", target="target", git=True
        )
        builder.build()
        self.assertTrue(Path("target/test").is_dir())
        self.assertTrue(Path("target/README.md").is_file())
        self.assertIsInstance(git.Repo("target"), git.Repo)


    def test_exception_handling(self):
        builder = boiler_builder(
            source=INVAILD_JSON, target="target"
        )
        self.assertEqual(
            builder.structure, format_project_structure(templates["python"],"target")
        )

        builder = boiler_builder(
            template="invalid", target="target"
        )
        self.assertEqual(
            builder.structure, format_project_structure(templates["python"],"target")
        )

        builder = boiler_builder(target="target")
        self.assertEqual(
            builder.structure, format_project_structure(templates["python"],"target")
        )



if __name__ == "__main__":
    pass



import unittest
from artec.exceptions import (
    NotJsonFile,
    NoSource,
    NotValidJson,
    InValidTemplate,
    GitError,
)


class TestExceptions(unittest.TestCase):
    def test_not_json_file(self):
        exception = NotJsonFile()
        self.assertEqual(exception.errno, 101)
        self.assertEqual(
            str(exception), "> Provided Source isn't valid JSON file"
        )

    def test_no_source(self):
        exception = NoSource()
        self.assertEqual(exception.errno, 102)
        self.assertEqual(str(exception), "> No Source Provided")

    def test_not_valid_json(self):
        exception = NotValidJson()
        self.assertEqual(exception.errno, 103)
        self.assertEqual(
            str(exception), "> Provided Json file format is incorrect"
        )

    def test_invalid_template(self):
        exception = InValidTemplate()
        self.assertEqual(exception.errno, 104)
        self.assertEqual(
            str(exception), "> Provided Template argument is invalid"
        )

    def test_git_error(self):
        exception = GitError()
        self.assertEqual(exception.errno, 105)
        self.assertEqual(str(exception), "> Git has encountered a problem")


if __name__ == "__main__":
    pass
